#include <mem.h>
#include <clarion.h>
#include <cla_file.h>
#include <cla_key.h>
#include <cla_err.h>
#include <key_stru.h>
#include <cla_mem.h>

static DBTYPES TypeEncode( int2 tp ) {
	switch ( tp ) {
		case 1: // Long
		return DBLONG;
		case 2: // Real
		return DBREAL;
		case 3: // String
		return DBSTRING;
		case 4: // Picture
		return DBSTRING;
		case 5: // Byte
		return DBBYTE;
		case 6: // Short
		return DBSHORT;
		case 7: // Group
		return DBNOTYPE;
		case 8: // Decimal
		return DBDECIMAL;
		default:
		return DBNOTYPE;
	}
}

TClaFile::TClaFile() : ISAMFile(), nRand( 0 ), nSeq( 0 ),
		pd( 0 ), keybuf( 0 ) , ClaErr( 0 ), nKey( 0 ),
		direction( 0 ), access( 0 ) {}

TClaFile::~TClaFile() {
	Close();
	delete pd;
	delete [] keybuf;
}

char * TClaFile::GetErrorString() {
	const l=60;
	static char s[l];
	cla_errstring( ClaErr, s, l );
	return s;
}

Boolean TClaFile::FieldByName( WORD* n, char* s ) {
	ClaErr=cla_fldnum( pd, s, (int2*)n );
	return ErrEncode();
}

Boolean TClaFile::KeyByName( WORD* n, char* s ) {
	ClaErr=cla_keynum( pd, s, (int2*)n );
	return ErrEncode();
}

Boolean TClaFile::NameByField( char* pch, WORD len, WORD n ) {
	ClaErr=cla_keyname( pd, n, pch, len );
	return ErrEncode();
}

Boolean TClaFile::NameByKey( char* pch, WORD len, WORD n ) {
	ClaErr=cla_keyname( pd, n, pch, len );
	return ErrEncode();
}

Boolean TClaFile::GetRecordLen( WORD* rl ) {
	ClaErr=cla_recbufsize( pd, rl );
	return ErrEncode();
}

Boolean TClaFile::GetFieldDescriptor( FieldDescriptor* fd, WORD n ) {
	if ( fd && pd && pd->hdr_field && n>0 && n<=pd->hdr->numflds ) {
		pcla_hdr_field pf=&pd->hdr_field[n-1];
		fd->Order=n;
		fd->Len=pf->length;
		fd->Precision=pf->decdec;
		fd->Type=TypeEncode( pf->fldtype );
		ClaErr=0;
	} else ClaErr=ERR_INVALID_POINTER;
	return ErrEncode();
}

Boolean TClaFile::Open( char* s ) {
	if ( !pd ) {
		pd=new cla_filedescr;
		memset( pd, 0, sizeof(cla_filedescr) );
	}
	ClaErr=cla_open( pd, s, 0 );
	if ( !ClaErr ) SetFirst();
	return ErrEncode();
}

Boolean TClaFile::Close() {
	ClaErr=cla_close( pd );
	SetFirst();
	return ErrEncode();
}

Boolean TClaFile::Create( char* /*fname*/ ) {
	ClaErr=ERR_UNDEFINED;
	return ErrEncode();
}

int2 TClaFile::KeyLen( int2 n ) {
	if ( pd && pd->hdr ) {
		int2 l=0;
		if ( n ) {
			if ( pd->keys && n>0 && n<=pd->hdr->numbkeys ) {
				l=pd->keys[n-1].keysect->complen;
			} else ClaErr=ERR_NOT_FOUND;
		}
		return l;
	} else ClaErr=ERR_INVALID_POINTER;
	return 0;
}

Boolean TClaFile::SetKey( int2 key, int2 dir, void* recbuf, uint4 rec ) {
	if ( key>=0 && key!=nKey ) {
		nKey=key;
		delete [] keybuf;
		keybuf=0;
		if ( nKey ) {
			int2 l=KeyLen( nKey );
			if ( l ) keybuf=new char[l];
		}
	}
	nSeq=rec;
	direction=dir;
	if ( recbuf && keybuf ) {
		ClaErr=key_fill( pd, nKey, recbuf, keybuf );
	}
	return ErrEncode();
}

Boolean TClaFile::Get( void* recbuf, DWORD rec, WORD key ) {
	if ( recbuf ) {
		void* keybuf=0;
		if ( key ) {
			int2 l=KeyLen( key );
			if ( !ClaErr ) {
				keybuf=new char[l];
				ClaErr=key_fill( pd, key, recbuf, keybuf );
			}
		}
		if ( !ClaErr ) Get( key, keybuf, recbuf, &rec, GET_EQ );
		if ( !ClaErr ) { access=1; nRand=rec; }
		delete keybuf;
	} else ClaErr=ERR_INVALID_POINTER;
	return ErrEncode();
}

Boolean TClaFile::GetSeq( void* recbuf, int2 dir ) { // dir: 0-next, 1-prev
	Boolean b=Get( nKey, keybuf, recbuf, &nSeq,
			(dir ? GET_LESS : GET_MORE)|
			(dir==direction ? 0 : GET_EQ) );
	direction=dir;
	if ( b ) {
		access=0;
		if ( keybuf ) {
			ClaErr=key_fill( pd, nKey, recbuf, keybuf );
		}
	}
	return ErrEncode();
}

Boolean TClaFile::Get( int2 key, void* keybuf, void* recbuf, uint4* rec, int2 action ) {
	if ( pd && pd->hdr && recbuf && rec ) {
		uint4 r=*rec;
		if ( key ) {
			if ( key>0 && key<=pd->hdr->numbkeys ) {
				ClaErr=key_find( pd->keys[key-1].hf, keybuf, &r, action );
			}
		}
		if ( !ClaErr ) ClaErr=dat_recget_phys( pd, r, recbuf );
		if ( !ClaErr ) *rec=r;
	} else ClaErr=ERR_INVALID_POINTER;
	return ErrEncode();
}

Boolean TClaFile::ErrEncode() {
	if ( ClaErr ) {
		Error=INTERNAL;
	} else Error=NOERROR;
	return Error==NOERROR ? TRUE : FALSE;
}

//int key_find( int hf, void* keybuf, unsigned long* pn, int action );
//int dat_recget_phys( pcla_filedescr pd, unsigned long n, void* buf );
//int dat_recput_phys( pcla_filedescr pd, unsigned long n, void* buf );

Boolean TClaFile::Add( void* recbuf ) {
	LastRecord()=0;
	return Put( recbuf );
}

Boolean TClaFile::Put( void* recbuf ) {
	ClaErr=cla_put( pd, recbuf, &LastRecord() );
	return ErrEncode();
}

Boolean TClaFile::Delete() {
	ClaErr=cla_delete( pd, LastRecord() );
	return ErrEncode();
}

Boolean TClaField::Get( void* rec, void* addr ) {
	if ( fl && ((TClaFile*)fl)->pd && rec && addr ) {
		int2 err=cla_fldget( ((TClaFile*)fl)->pd, fd.Order, rec, addr );
		Error=err ? INTERNAL : NOERROR;
	} else Error=INTERNAL;
	return Error==NOERROR ? TRUE : FALSE;
}

Boolean TClaField::Set( void* rec, void *addr ) {
	if ( fl && ((TClaFile*)fl)->pd && rec && addr ) {
		int2 err=cla_fldput( ((TClaFile*)fl)->pd, fd.Order, rec, addr );
		Error=err ? INTERNAL : NOERROR;
	} else Error=INTERNAL;
	return Error==NOERROR ? TRUE : FALSE;
}

Boolean TClaFile::GetKeyDescriptor( KeyDescriptor* pkd, WORD n ) {
	if ( pkd ) {
		int2 nn=0;
		ClaErr=cla_keycount( pd, &nn );
		if ( !ClaErr ) {
			if ( n>0 && n<=nn ) {
				pkd->keyNum=n;
				pkd->numComp=pd->keys[n-1].keysect->numcomps;
				delete [] pkd->comp;
				pkd->comp=new WORD[pkd->numComp];
				pkd->isDup=pd->keys[n-1].keysect->comptype&KEYTYP_DUPSW ? TRUE : FALSE;
				pkd->isOpt=pd->keys[n-1].keysect->comptype&KEYTYP_OPTSW ? TRUE : FALSE;
				pkd->isUpp=pd->keys[n-1].keysect->comptype&KEYTYP_UPRSW ? TRUE : FALSE;
				pkd->isAcc=TRUE;
			} else ClaErr=ERR_NOT_FOUND;
		}
	} else ClaErr=ERR_INVALID_POINTER;
	return ErrEncode();
}

Boolean TClaFile::GetFileDescriptor( FileDescriptor* pfd ) {
	if ( pfd && pd ) {
		pfd->memo=pd->hdr->sfatr&SFATR_MEMO ? TRUE : FALSE;
		pfd->reclaim=pd->hdr->sfatr&SFATR_RECLAIM ? TRUE : FALSE;
		pfd->numbkeys=pd->hdr->numbkeys;   /* number of keys in file */
		pfd->numrecs=pd->hdr->numrecs;    /* number of records in file */
		pfd->numdels=pd->hdr->numdels;    /* number of deleted records */
		pfd->numflds=pd->hdr->numflds;         /* number of fields */
		pfd->reclen=pd->hdr->reclen;          /* record length (including record header) */
		pfd->memolen=pd->hdr->memolen;         /* size of memo */
		pfd->memowid=pd->hdr->memowid;         /* column width of memo */
	} else ClaErr=ERR_INVALID_POINTER;
	return ErrEncode();
}

Boolean TClaFile::FieldCount( WORD* pn ) {
	ClaErr=cla_fldcount( pd, (int2*)pn );
	return ErrEncode();
}

Boolean TClaFile::KeyCount( WORD* pn ) {
	ClaErr=cla_keycount( pd, (int2*)pn );
	return ErrEncode();
}

Boolean TClaFile::GetMemo( void* pm, WORD n, void* recbuf ) {
  uint4 nn=n;
  ClaErr=cla_memget( pd->hfile, ((pcla_hdr_rec)(recbuf))->rptr, &nn, pm );

	return ErrEncode();
}

Boolean TClaFile::SetMemo( void* pm, WORD n, void* recbuf ) {
  ClaErr=cla_memset( pd->hfile, &((pcla_hdr_rec)(recbuf))->rptr, n, pm );
	return ErrEncode();
}