#include <stdlib.h>     /* ldiv */
#include <dat.h>
#include <cla_dat.h>
#include <cla_err.h>

/* shift between clarion's & borland's data representation */
static const int4 magic=2378858L;

/* check is date valid, pointer should be non-0 */
static int2 is_date_valid( pcla_date pd ) {
	return (int2)(pd->year>1801 && pd->year<3000
			&& pd->day>0 && pd->day<=day_in_month( pd->month, pd->year ));
}

/* check is time valid, pointer should be non-0 */
static int2 is_time_valid( pcla_time pt ) {
	return (int2)(pt->hour>=0 && pt->hour<24
				&& pt->min>=0 && pt->min<60
				&& pt->sec>=0 && pt->sec<60
				&& pt->hundreds>=0 && pt->hundreds<100);
}

int2 cla_time_encode( pcla_time pt, int4* pxt ) {
	if ( pt && pxt ) {
		if ( is_time_valid( pt ) ) {
			*pxt=((pt->hour*60+pt->min)*60+pt->sec)*100+pt->hundreds+1;
			return ERR_NONE;
		} else return ERR_INVALID_TIME;
	} else return ERR_INVALID_POINTER;
}

int2 cla_time_decode( pcla_time pt, int4 xt ) {
	if ( pt ) {
		if ( xt>0 && xt<=8640000 ) {
			ldiv_t t=ldiv( --xt, 100 );
			pt->hundreds=(int2)t.rem;
			t=ldiv( t.quot, 60 );
			pt->sec=(int2)t.rem;
			t=ldiv( t.quot, 60 );
			pt->min=(int2)t.rem;
			pt->hour=(int2)t.quot;
			return ERR_NONE;
		} else return ERR_INVALID_TIME;
	} else return ERR_INVALID_POINTER;
}

/* 4 <--> 1/1/1801 */
int2 cla_date_encode( pcla_date pd, uint4* pxd ) {
	if ( pd && pxd ) {
		if ( is_date_valid( pd )
				&& 0==date_encode( pd->year, pd->month, pd->day, pxd ) ) {
			*pxd-=magic;
			return ERR_NONE;
		} else return ERR_INVALID_DATE;
	} else return ERR_INVALID_POINTER;
}

int2 cla_date_decode( pcla_date pd, uint4 xd ) {
	if ( pd ) {
		int2 d=0, m=0, y=0;
		if ( xd>3 && xd<109211
				&& 0==date_decode( &y, &m, &d, xd+magic ) ) {
			pd->year=y;
			pd->month=m;
			pd->day=d;
			return ERR_NONE;
		} else return ERR_INVALID_DATE;
	} else return ERR_INVALID_POINTER;
}
