#include <io1.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#ifdef __OS2__
#include <malloc.h>
#else
#include <alloc.h>
#endif
#include <cla_file.h>
#include <cla_err.h>
#include <cla_lock.h>
/*--------------------------------------------
int2 cla_open( pcla_filedescr pd, int1* fname, int2 repair );
int2 cla_close( pcla_filedescr pd );
int2 cla_refresh( pcla_filedescr pd, int2 wr );
 ---------------------------------------------*/

static int2 read_file_header( int2 hfile, pcla_hdr_file* pph ) {
	int2 err=ERR_NONE;
	pcla_hdr_file ph=(pcla_hdr_file)malloc( sizeof( cla_hdr_file ) );
	if ( ph ) {
		if ( 0==lseek( hfile, 0, SEEK_SET )
				&& sizeof( cla_hdr_file )==read( hfile, ph, sizeof( cla_hdr_file ) ) ) {
				/* now read all header structures ( till data ) */
			if ( ph->filesig==0x3343
					&& ph->offset>=sizeof( cla_hdr_file )
					&& ph->offset<0xffff ) {
				uint2 off=(uint2)( ph->offset );
				pcla_hdr_file ph1=ph;
				ph=(pcla_hdr_file)realloc( ph, off );
				if ( ph ) {
					if ( 0==lseek( hfile, 0, SEEK_SET )
							&& off==read( hfile, ph, off ) ) {
						/* OK */
					} else err=( err ? err : ERR_READ );
				} else {
					ph=ph1;
					err=err ? err : ERR_MEM;
				}
			} else err=err ? err : ERR_FILE_CORRUPTED;
		} else err=err ? err : ERR_READ;
	} else err=err ? err : ERR_MEM;
	if ( err ) {
		free( ph );
	} else {
		*pph=ph;
	}
	return err;
}

static int1 h2c( uint2 h ) {
	return h>9 ? h+'a'-10 : h+'0';
}

/* open file, fill descriptor */
int2 cla_open( pcla_filedescr pd, int1* fname, int2 repair ) {
	if ( pd && fname ) {
		int2 err=cla_close( pd );
		if ( !err && (pd->hfile=open( fname, O_BINARY | O_RDWR, 0 ))>0 ) {
			err=read_file_header( pd->hfile, &(pd->hdr) );
			/* parse header parts */
			if ( !err ) {
				/* store hdr_field, ptr point to part after fields */
				int1* ptr=(int1*)&((pd->hdr_field=(pcla_hdr_field)&(pd->hdr[1]))[pd->hdr->numflds]);
				/* memo */
				if ( pd->hdr->sfatr&SFATR_MEMO ) {
					int1* smem=(int1*)malloc( strlen( fname )+4+1 ); /* fname + ".K01" */
					if ( smem ) {
						int1* dot=strrchr( strcpy( smem, fname ), '.' );
						if ( dot ) dot++;
						else {
							dot=smem+strlen( smem );
							*dot++='.';
						}
						*dot++='m';
						*dot++='e';
						*dot++='m';
						*dot='\0';
						pd->hmemo=open( smem, O_BINARY | O_RDWR, 0 );
						if ( pd->hmemo<0 && !(repair&REPAIR_KEEP_BROKEN) ) {
							if ( repair&REPAIR_MEMO ) {
								/* create memo */
								/*             */
							} else {
								err=ERR_FILE_CORRUPTED;
							}
						}
						free( smem );
					} err=ERR_MEM;
				}
				if ( !err && pd->hdr->numbkeys ) { /* there are some keys */
					int2 n=(int2)pd->hdr->numbkeys;
					int1* skey=(int1*)malloc( strlen( fname )+4+1 ); /* fname + ".K01" */
					pd->keys=(pcla_keydescr)malloc( sizeof(cla_keydescr)*n );
					if ( skey && pd->keys ) {
						int2 i=0;
						/* prepare string for key file name */
						int1* dot=strrchr( strcpy( skey, fname ), '.' );
						if ( dot ) dot++;
						else {
							dot=skey+strlen( skey );
							*dot++='.';
						}
						*dot++='k';
						dot[2]='\0';
						for ( ;!err && i<n; i++ ) {
							dot[0]=h2c( (i+1)/16 );
							dot[1]=h2c( (i+1)%16 );
							pd->keys[i].keysect=(pcla_keysect)ptr;
							pd->keys[i].hf=open( skey, O_BINARY | O_RDWR, 0 );
							if ( pd->keys[i].hf<0 && !(repair&REPAIR_KEEP_BROKEN) ) {
								if ( repair&REPAIR_KEY ) {
									/* create key */
									/*            */
								} else {
									err=ERR_FILE_CORRUPTED;
								}
							}
							ptr+=sizeof(cla_keysect)+
									((pcla_keysect)ptr)->numcomps*sizeof(cla_keypart);
						}
					} else {
						err=ERR_MEM;
					}
					free( skey );
				}
				if ( !err && pd->hdr->numpics ) {
					pd->hdr_pict=(pcla_hdr_pict)ptr;
					ptr=(int1*)&(pd->hdr_pict[pd->hdr->numpics]);
				}
				if ( !err && pd->hdr->nummars ) {
					int2 n=pd->hdr->nummars;
					pd->arrays=(pcla_arrsect*)malloc( sizeof(pcla_arrsect)*n );
					if ( pd->arrays ) {
						int2 i=0;
						for ( ; i<n; i++ ) {
							pd->arrays[i]=(pcla_arrsect)ptr;
							ptr+=sizeof(cla_arrsect)+
									((pcla_arrsect)ptr)->totdim*sizeof(cla_arrpart);
						}
					} else err=ERR_MEM;
				}
				if ( !err ) {
					/*pd->fname=strdup( fname ); non-ANSI*/
					pd->fname=strcpy( (int1*)malloc( strlen( fname )+1 ), fname );
					if ( !pd->fname ) err=ERR_MEM;
				}
			}
		} else err=err ? err : ERR_OPEN;
		if ( err ) {
			cla_close( pd );
		}
		return err;
	} else return ERR_INVALID_POINTER;
}

int2 cla_close( pcla_filedescr pd ) {
	if ( pd ) {
		int2 err=0;
		if ( pd->hfile>0 ) err=close( pd->hfile );
		if ( pd->hmemo>0 ) err=(0==close( pd->hmemo )) ? err : -1;
		if ( pd->keys ) {
			int2 i=0;
			for ( ; i<pd->hdr->numbkeys; i++ ) {
				int hf=pd->keys[i].hf;
				if ( hf>0 ) err=(0==close( hf )) ? err : -1;
			}
		}
		if ( pd->hdr ) free( pd->hdr );
		if ( pd->fname ) free( pd->fname );
		if ( pd->keys ) free( pd->keys );
		if ( pd->arrays ) free( pd->arrays );
		memset( pd, 0, sizeof(cla_filedescr) );
		return err ? ERR_CLOSE : ERR_NONE;
	} else return ERR_INVALID_POINTER;
}
/* make file header in descriptor up-to-date */
int2 cla_refresh( pcla_filedescr pd, int2 wr ) {
	if ( pd ) {
		if ( pd->hfile>0 && pd->hdr ) {
			if ( 0==lseek( pd->hfile, 0, SEEK_SET )
					&& sizeof( cla_hdr_file )==( wr ? write( pd->hfile, pd->hdr, sizeof( cla_hdr_file ) ) : read( pd->hfile, pd->hdr, sizeof( cla_hdr_file ) ) ) ) {
				return ERR_NONE;
			} else return ERR_READ;
		} else return ERR_NOT_FOUND;
	} else return ERR_INVALID_POINTER;
}

static int2 cla_lock_all( pcla_filedescr pd ) {
	int2 err=0;
	if ( pd->hfile>0 ) err=cla_lock( pd->hfile );
	if ( !err ) {
		if ( pd->hmemo>0 ) err=cla_lock( pd->hmemo );
		if ( !err ) {
			if ( pd->keys ) {
				int2 i=0;
				for ( ; i<pd->hdr->numbkeys && !err; i++ ) {
					int hf=pd->keys[i].hf;
					if ( hf>0 ) err=cla_lock( hf );
				}
				if ( err ) {
					for ( i-=2; i>=0; i-- ) {
						int hf=pd->keys[i].hf;
						if ( hf>0 ) err=cla_unlock( hf );
					}
				}
			}
			if ( err ) cla_unlock( pd->hmemo );
		}
		if ( err ) cla_unlock( pd->hfile );
	}
	return err ? ERR_LOCK_FAIL : ERR_NONE;
}

static int2 cla_unlock_all( pcla_filedescr pd ) {
	int2 err=0, err1=0;
	if ( pd->hfile>0 ) err=cla_unlock( pd->hfile );
	if ( pd->hmemo>0 ) err1=cla_unlock( pd->hmemo );
	if ( !err ) err=err1;
	if ( pd->keys ) {
		int2 i=0;
		for ( ; i<pd->hdr->numbkeys; i++ ) {
			int hf=pd->keys[i].hf;
			if ( hf>0 ) err1=cla_unlock( hf );
			if ( !err ) err=err1;
		}
	}
	return err ? ERR_UNLOCK_FAIL : ERR_NONE;
}

int2 cla_lockfamily( pcla_filedescr pd, int2 lck ) {
	if ( pd ) {
		return lck ? cla_lock_all( pd ) : cla_unlock_all( pd );
	} else return ERR_INVALID_POINTER;
}