#ifndef __CLA_FILE_H
#define __CLA_FILE_H
#include <fix_type.h>
#include <cla_stru.h>
#ifdef __cplusplus
  extern "C" {
#endif

typedef struct str_cla_keydescr {
	pcla_keysect keysect;     /* point into 'hdr' allocated in file descriptor */
	int          hf;
} cla_keydescr;
typedef cla_keydescr* pcla_keydescr;

/* file descriptor */
typedef struct str_cla_filedescr {
	pcla_hdr_file hdr;         /* really allocated */
	pcla_hdr_field hdr_field;  /*---------------------*/
	/*pcla_hdr_key   hdr_key;*/    /* this 4 pointers are */
	pcla_hdr_pict  hdr_pict;   /* allocated in 'hdr'  */
	/*pcla_hdr_arr   hdr_arr;*/    /*---------------------*/

	int1* fname;               /* malloc/free when open/close */
	int2 hfile;
	/* linked files : memo & keys */
	int2 hmemo;                 /* memo file */

	pcla_keydescr keys;       /* array of keys  , malloc/free */
	pcla_arrsect*  arrays;     /* array of arrays, malloc/free */
	/* access, current state */
	/* current key - index in 'keys', starting with 1 , 0 - none */
	int2            current_key;

/*	int2            last_kind;*/  /* kind of last access : 0-random, 1-seq */
/*	uint4  n_random;*/
	/* get_random, put_random */
/*	uint4  n_seq;*/
	/* next : [++n_seq] */
	/* prev : [--n_seq] */
	/* put  : [n_seq++] */
} cla_filedescr;
typedef cla_filedescr* pcla_filedescr;
/* open/close */
/* open .dat, indexes & memo, fill descriptor.
 'fd' should be valid descriptor of previously opened file, or
 0 - filled. If 'fd' contain file descriptor, they will close.
 'repair' is bit field parameter which describe actions performed
 in a case of broken family files.
*/
/* open file, fill descriptor. Indexes & memo are NOT opened */
/* repair broken keys */
#define REPAIR_KEY (0x01)
/* repair broken memo */
#define REPAIR_MEMO (0x02)
/* keep all broken parts unopened */
#define REPAIR_KEEP_BROKEN (0x80)

int2 cla_open( pcla_filedescr fd, int1* fname, int2 repair );
/* close file and key, if any */
int2 cla_close( pcla_filedescr );
/* make file header in descriptor up-to-date
	 if wr<>0, write, otherwise read */
int2 cla_refresh( pcla_filedescr, int2 wr );

/*-----------Key number-------------
 All functions that have key number as parameter,
 use the following convention:
 -1 : default, or current, key
 0  : no key
 >0 : key number
 ----------------------------------*/

int2 cla_keycount( pcla_filedescr pd, int2* pcount);/* get total number of keys */
int2 cla_fldcount( pcla_filedescr pd, int2* pcount);/* get total number of fields */
int2 cla_keysize( pcla_filedescr, int2 nkey, int2* psz );/* size of key buffer */
int2 cla_setkey( pcla_filedescr pd, int2 nkey );
/* fill key buffer from values in record buffer */
int2 key_fill( pcla_filedescr pd, int2 nkey, void* recbuf, void* keybuf );
/* find key name by number (1..) */
int2 cla_keyname( pcla_filedescr pd, int2 n, int1* pch, int2 len );
/* find key number by name */
int2 cla_keynum( pcla_filedescr pd, int1* pch, int2* pn );

int2 cla_recbufsize( pcla_filedescr pd, uint2* psz );/* including header */
int2 key_create( pcla_filedescr pd, int2 nkey );
int2 cla_put( pcla_filedescr pd, void* recbuf, uint4* rec );
int2 cla_delete( pcla_filedescr pd, uint4 rec );
/*=======================================================*/
/*
 get record number( phys ) of specified index & action
 >>action<<
 bit 0    : 0-exact match, 1-exact or greater
 bits 1-2 : 0-get this, 1-get next, 2-get prev
 */
/*
int2 cla_keyfind( pcla_filedescr pd, void* buf, uint4* pn, int2 action );
*/
int2 dat_recget_phys( pcla_filedescr pd, uint4 n, void* buf );
int2 dat_recput_phys( pcla_filedescr pd, uint4 n, void* buf );
int2 dat_freerec( pcla_filedescr pd, uint4* rec );
int2 dat_isrecempty( pcla_filedescr pd, int2 nkey, void* recbuf, int2* pe );

int2 cla_fldname( pcla_filedescr pd, int2 n, int1* pch, int2 len );
int2 cla_fldnum( pcla_filedescr pd, int1* pch, int2* pn );
int2 cla_fldtype( pcla_filedescr, int2, int1* ); /* [10] */
int2 cla_fldget( pcla_filedescr pd, int2 n, void* buf, void* addr );
int2 cla_fldput( pcla_filedescr pd, int2 n, void* buf, void* addr );
int2 cla_lockfamily( pcla_filedescr pd, int2 lck );

#ifdef __cplusplus
  }
#endif
#endif /* __CLA_FILE_H */
