#include <ctype.h>
#include <string.h>
#ifdef __OS2__
#include <malloc.h>
#else
#include <alloc.h>
#endif
#include <key_util.h>
#include <cla_key.h>
#include <cla_err.h>
/*-----------------------------------------------------------------
 int2 key_order( int hf, uint4 ord, uint4* pn );
 int2 key_find( int hf, void* buf, uint4* pn, int2 action );
 -----------------------------------------------------------------*/
/* find physical number for record number 'ord'
 in sequence of index in file 'hf' */
int2 key_order( int hf, uint4 ord, uint4* pn ) {
	if ( hf>0 && pn ) {
		uint4 rec=0;
		pkey_hdr ph=(pkey_hdr)malloc( sizeof( key_hdr ) );
		pkey_node pnode=(pkey_node)malloc( sz_node );
		int2 err=key_gethdr( hf, ph );
		if ( !err && ph && pnode ) {
			if ( ord>0 && ord<=ph->numkent ) {
				/* root */
				err=key_getnode( hf, ph->root, pnode );
				/* first leaf */
				{
					uint2 lvl=1;
					while ( !err && lvl<ph->numlvls ) {
						err=key_getnode( hf, ((pkey_entry)&pnode[1])->relrec, pnode );
						lvl++;
					}
				}
				/* now we are at 1-st leaf */
				/* iterate by leafs */
				while ( !err && ord>pnode->keycnt ) {  /* 1..keycnt (inclusive) */
					ord-=pnode->keycnt;
					err=key_next_node( hf, pnode );
				}
				/* in this leaf */
				if ( !err ) {
					pkey_entry pe=(pkey_entry)&(((int1*)(&pnode[1]))[(int2)(ord-1)*ph->keylen]);
					rec=pe->relrec;
				}
			} else err=ERR_INVALID_POINTER;
		} else err=err ? err : ERR_MEM;
		free( pnode );
		free( ph );
		if ( !err ) *pn=rec;
		return err;
	} else return ERR_INVALID_POINTER;
}

static int2 confirm_entry( int hf, pkey_node pnode, int2* pi ) {
	int2 err=0;
	if ( *pi<0 ) {
		err=key_prev_node( hf, pnode );
		if ( !err ) *pi=pnode->keycnt-1;
	} else if ( *pi>=pnode->keycnt ) {
		err=key_next_node( hf, pnode );
		if ( !err ) *pi=0;
	}
	return err;
}

int2 key_find( int hf, void* buf, uint4* pn, int2 action ) {
	if ( hf>0 && buf && pn ) {
		int2 i=0, cmp=0;
		uint4 nn=0;
		pkey_node pnode=(pkey_node)malloc( sz_node );
		/* header is the same size, so use them to obtain keylen */
		int2 err=key_gethdr( hf, (pkey_hdr)pnode );
		int2 keylen=((pkey_hdr)pnode)->keylen;
		if ( !err ) err=key_findnode( hf, buf, *pn, pnode, &nn, &i, &cmp );
		nn=0; /* nn have another meaning and used here for dummy */
		if ( !err ) {
			/* base record */
			if ( ( cmp==0 && (action&GET_EQ) )
					||( cmp<0 && (action&GET_MORE) )
					||( cmp>0 && (action&GET_LESS) ) ) {
				nn=((pkey_entry)&(((int1*)(&pnode[1]))[i*keylen]))->relrec;
				/* already found */
			} else if ( cmp==0 && action==GET_MORE ) {
				/* next */
				i++;
				err=confirm_entry( hf, pnode, &i );
				if ( !err ) nn=((pkey_entry)&(((int1*)(&pnode[1]))[i*keylen]))->relrec;
			} else if ( cmp<=0 && (action&GET_LESS) ) {
				/* prev */
				i--;
				err=confirm_entry( hf, pnode, &i );
				if ( !err ) nn=((pkey_entry)&(((int1*)(&pnode[1]))[i*keylen]))->relrec;
			} else err=ERR_NOT_FOUND;
		}
		if ( !err ) {
			*pn=nn;
		}
		free( pnode );
		return err;
	} else return ERR_INVALID_POINTER;
}
