#include <io1.h>
#include <fcntl.h>
#include <string.h>
#ifdef __OS2__
#include <malloc.h>
#else
#include <alloc.h>
#endif
#include <sys/stat.h>
#include <key_util.h>
#include <cla_key.h>
#include <cla_file.h>
#include <cla_err.h>
/*------------------------------------------------
int2 key_create( pcla_filedescr pd, int2 i );
--------------------------------------------------*/

static int1 h2c( uint2 h ) {
	return (int1)(h>9 ? h+'a'-10 : h+'0');
}

/* make key file name. allocate string using 'malloc' */
static int1* key_fname( int1* fname, int2 nkey ) {
	if ( fname ) {
		int1* skey=(int1*)malloc( strlen( fname )+4+1 ); /* fname + ".K01" */
		if ( skey ) {
			int1* dot=strrchr( strcpy( skey, fname ), '.' );
			if ( dot ) dot++;
			else {
				dot=skey+strlen( skey );
				*dot++='.';
			}
			*dot++='k';
			*dot++=h2c( (uint2)(nkey/16) );
			*dot++=h2c( (uint2)(nkey%16) );
			*dot='\0';
			return skey;
		}
	}
	return 0;
}

static int2 key_createempty( pcla_keydescr pkd, int1* skey ) {
	pkey_hdr ph=(pkey_hdr)malloc( sizeof(key_hdr) );
	if ( ph ) {
		int2 err=0;
		int hf=open( skey, O_BINARY | O_CREAT | O_RDWR | O_EXCL, S_IREAD|S_IWRITE );
		if ( hf>0 ) {
			memset( ph, 0, sizeof(key_hdr) );
			ph->root=1;
			ph->numnode=1;
			ph->lastnod=1;
			ph->numlvls=1;
			ph->numcmps=pkd->keysect->numcomps;
			ph->keylen=(uint2)(pkd->keysect->complen+4);
			ph->keynode=(uint1)(sz_node/ph->keylen);
			ph->keytyp=pkd->keysect->comptype;
			err=key_puthdr( hf, ph );
			if ( !err ) {
				memset( ph, 0, sizeof(key_hdr) );
				err=key_putnode( hf, 1, (pkey_node)(void*)ph );
			}
			close( hf );
		} else {
			err=ERR_OPEN;
			/* perror( "\nKey_createempty" ); */
		}
		free( ph );
		return err;
	} else return ERR_MEM;
}

int2 key_create( pcla_filedescr pd, int2 i ) {
	if ( pd && pd->hdr && pd->keys && i>0 && i<=pd->hdr->numbkeys ) {
		pcla_keydescr pkd=&pd->keys[i-1];
		int1* skey=key_fname( pd->fname, i );
		if ( skey ) {
			int2 err=key_createempty( pkd, skey );
			if ( !err ) {
				uint4 r=1;
				void *rbuf=malloc( pd->hdr->reclen ),
				*kbuf=malloc( (size_t)(pkd->keysect->complen+4) );
				pkd->hf=open( skey, O_BINARY | O_RDWR, 0 );
				if ( pkd->hf<0 ) err=ERR_OPEN;
#if 1   /* normal way - one pass, add only non-deleted */
				while ( !err && r<=pd->hdr->numrecs ) {
					err=dat_recget_phys( pd, r, rbuf );
					if ( err==ERR_DELETED ) {
						err=0;
					} else {
						if ( !err ) err=key_fill( pd, i, rbuf, kbuf );
						if ( !err ) err=key_insert( pkd->hf, kbuf, r );
					}
					r++;
				}
#else /* debug - 2 passes, 1st - add all, 2-nd - delete deleted */
				while ( !err && r<=pd->hdr->numrecs ) {
					err=dat_recget_phys( pd, r, rbuf );
					if ( err==ERR_DELETED ) err=0;
					if ( !err ) err=key_fill( pd, i, rbuf, kbuf );
					if ( !err ) err=key_insert( pkd->hf, kbuf, r );
					r++;
				}
				r=1;
				while ( !err && r<=pd->hdr->numrecs ) {
					err=dat_recget_phys( pd, r, rbuf );
					if ( err==ERR_DELETED ) {
						err=0;
						if ( !err ) err=key_fill( pd, i, rbuf, kbuf );
						if ( !err ) err=key_( pkd->hf, kbuf, r );
					}
					r++;
				}

#endif
				free( kbuf );
				free( rbuf );
			}
			free( skey );
			return err;
		} else return ERR_MEM;
	} else return ERR_INVALID_POINTER;
}
