#include <io1.h>
#include <ctype.h>
#ifdef __OS2__
#include <string.h>
#include <malloc.h>
#else
#include <mem.h>
#include <alloc.h>
#endif
#include <cla_file.h>
#include <cla_str.h>
#include <cla_err.h>
/*-----------------------------------------
int2 dat_recget_phys( pcla_filedescr pd, uint4 n, void* buf );
int2 dat_recput_phys( pcla_filedescr pd, uint4 n, void* buf );
int2 key_fill( pcla_filedescr pd, int2 nkey, void* recbuf, void* keybuf );
int2 dat_isrecempty( pcla_filedescr pd, int2 nkey, void* recbuf, int* pe );
 ------------------------------------------*/
static int4 rec_off( pcla_filedescr pd, uint4 n ) {
	return (int4)(pd->hdr->offset+(n-1)*pd->hdr->reclen);
}

int2 dat_recget_phys( pcla_filedescr pd, uint4 n, void* buf ) {
	if ( pd && buf && pd->hdr ) {
		int4 pos=rec_off( pd, n );
		uint2 rl=pd->hdr->reclen;
		if ( pos==lseek( pd->hfile, pos, SEEK_SET )
				&& rl==read( pd->hfile, buf, rl ) ) {
			return ((pcla_hdr_rec)buf)->rhd&RHD_IS_DEL ? ERR_DELETED :
				((pcla_hdr_rec)buf)->rhd&RHD_IS_HLD ? ERR_LOCKED : ERR_NONE;
		} else return ERR_READ;
	} else return ERR_INVALID_POINTER;
}

int2 dat_recput_phys( pcla_filedescr pd, uint4 n, void* buf ) {
	if ( pd && buf && pd->hdr ) {
		int4 pos=rec_off( pd, n );
		uint2 rl=pd->hdr->reclen;
		if ( pos==lseek( pd->hfile, pos, SEEK_SET )
				&& rl==write( pd->hfile, buf, rl ) ) {
			return ERR_NONE;
		} else return ERR_READ;
	} else return ERR_INVALID_POINTER;
}

/* fill key buffer with field value,
 converted corresp. to they type */
static int2 fld2key( pcla_filedescr pd, int2 n, pcla_keysect pk, void* buf, void* addr ) {
	if ( pd && buf && addr ) {
		if ( pd->hdr && n>0 && n<=pd->hdr->numflds ) {
			pcla_hdr_field pf=&pd->hdr_field[n-1];
			uint1* src=(uint1*)buf+sizeof( cla_hdr_rec )+pf->foffset;
			switch ( pf->fldtype ) {
				case 1:  /* LONG */
				((uint1*)addr)[3]=src[0];
				((uint1*)addr)[2]=src[1];
				((uint1*)addr)[1]=src[2];
				((uint1*)addr)[0]=src[3]^0x80;
				break;
				case 2:  /* REAL */
				{
					int2 i=8;
					while ( i-- ) ((uint1*)addr)[i]=src[7-i];
				 /*	if ( ((int1*)addr)[0]<0 ) */ ((uint1*)addr)[0]^=0x80;
				}
				break;
				case 3:  /* STRING */
				case 4:  /* STRING WITH PICTURE TOKEN */
				case 7:  /* GROUP */
				if ( pk->comptype & 0x20 ) { /* uppercase */
					int2 i=pf->length;
					while ( i-- ) ((uint1*)addr)[i]=toupper( src[i] );
				} else {
					memcpy( addr, src, pf->length );
				}
				break;
				case 5:  /* BYTE */
				((int1*)addr)[0]=src[0];
				break;
				case 6:  /* SHORT */
				((uint1*)addr)[1]=src[0];
				((uint1*)addr)[0]=src[1]^0x80;
				break;
				case 8:  /* DECIMAL */
				{
					int2 i=pf->length;
					if ( src[0]<0 ) {
						while ( i-- ) ((uint1*)addr)[i]=~src[i];
					} else {
						memcpy( addr, src, pf->length );
						((uint1*)addr)[0]|=0x80;
					}
				}
				break;
			}
			return ERR_NONE;
		} else return ERR_NOT_FOUND;
	} else return ERR_INVALID_POINTER;
}

int2 key_fill( pcla_filedescr pd, int2 nkey, void* recbuf, void* keybuf ) {
	if ( pd && pd->hdr && pd->keys && recbuf && keybuf ) {
		if ( nkey==-1 ) nkey=pd->current_key;
		if ( nkey>0 && nkey<=pd->hdr->numbkeys ) {
			int2 ncomp=pd->keys[nkey-1].keysect->numcomps;
			pcla_keysect ps=pd->keys[nkey-1].keysect;
			pcla_keypart pp=(pcla_keypart)(void*)&(ps[1]);
			int2 i=0;
			int2 err=0;
			int1* addr=(int1*)keybuf;
			for ( ; i<ncomp; i++ ) {
				err=fld2key( pd, pp[i].fldnum, ps, recbuf, addr );
				if ( err ) return err;
				addr+=pp[i].elmlen;
			}
			return ERR_NONE;
		} else return ERR_NOT_FOUND;
	} else return ERR_INVALID_POINTER;
}

int2 dat_freerec( pcla_filedescr pd, uint4* rec ) {
	int2 err=0;
	if ( pd->hdr->freerec && pd->hdr->sfatr&SFATR_RECLAIM ) {
		int1* recbuf=(int1*)malloc( pd->hdr->reclen );
		err=dat_recget_phys( pd, pd->hdr->freerec, recbuf );
		if ( err==ERR_DELETED ) err=0;
		else if ( !err ) err=ERR_NOT_FOUND;
		if ( !err ) {
			*rec=pd->hdr->freerec;
			pd->hdr->freerec=((pcla_hdr_rec)recbuf)->rptr;
			pd->hdr->numdels--;
		}
		free( recbuf );
	} else {
		*rec=++pd->hdr->numrecs;
	}
	return err;
}

int2 dat_isrecempty( pcla_filedescr pd, int2 nkey, void* recbuf, int2* pe ) {
	int2 err=0;
  if ( pd && pd->hdr && recbuf && pd->keys ) {
		if ( nkey==-1 ) nkey=pd->current_key;
		if ( nkey>0 && nkey<=pd->hdr->numbkeys ) {
			int2 ncomp=pd->keys[nkey-1].keysect->numcomps;
			pcla_keysect ps=pd->keys[nkey-1].keysect;
			pcla_keypart pp=(pcla_keypart)(void*)&(ps[1]);
			int2 i=0;
      int2 empty=1;
			for ( ; i<ncomp && empty && !err; i++ ) {
  			pcla_hdr_field pf=&pd->hdr_field[i];
        void* src=(int1*)recbuf+sizeof( cla_hdr_rec )+pf->foffset;
			  switch ( pf->fldtype ) {
				  case 1:  /* LONG */
          empty=(*((int4*)src)==0);
  				break;
		  		case 2:  /* REAL */
          empty=(*((float*)src)==0);
  				break;
		  		case 3:  /* STRING */
				  case 4:  /* STRING WITH PICTURE TOKEN */
  				case 7:  /* GROUP */
          empty=(cla_strlen( src, pf->length )==0);
  				break;
		  		case 5:  /* BYTE */
          empty=(*((int1*)src)==0);
  				break;
		  		case 6:  /* SHORT */
          empty=(*((int2*)src)==0);
  				break;
		  		case 8:  /* DECIMAL */
          { 
            int i=0;
            for ( ; i<pf->length; i++ ) {
              if ( ((int1*)src)[i] ) {
                empty=0;
                break;
              }
            } /* endfor */
          }
  				break;
		  	}
			}
      if ( !err ) {
        *pe=empty;
      }
		} else err=ERR_NOT_FOUND;
  } else err=ERR_INVALID_POINTER;
	return err;
}
