#ifdef __OS2__
#include <malloc.h>
#else
#include <alloc.h>
#endif
#include <cla_file.h>
#include <cla_key.h>
#include <cla_err.h>
#include <key_stru.h>
/*---------------------------------------------------
int2 cla_put( pcla_filedescr pd, void* recbuf, uint4* rec );
int2 cla_delete( pcla_filedescr pd, uint4 rec );
-----------------------------------------------------*/
/* add to ( del==0 ) or delete from all keys */
static int2 cla_allkeys( pcla_filedescr pd, void* recbuf, uint4 rec, uint2 del ) {
	int2 err=0;
	if ( pd->keys ) {
		int2 i=0;
		void* keybuf=malloc( pd->hdr->reclen );
		for ( ; i<pd->hdr->numbkeys && (!err || del&0x02); i++ ) {
			int hf=pd->keys[i].hf;
			if ( hf>0 ) {
        int2 empty=0;
        if ( !err && pd->keys[i].keysect->comptype&KEYTYP_OPTSW ) err=dat_isrecempty( pd, (int2)(i+1), recbuf, &empty );
        if ( !empty ) {
  				if ( !err ) err=key_fill( pd, (int2)(i+1), recbuf, keybuf );
		  		if ( !err ) err=( del&0x01 ? key_delete( hf, keybuf, rec ) : key_insert( hf, keybuf, rec ) );
        }
			}
		}
		free( keybuf );
	}
	return err;
}

static int2 cla_queryunique( pcla_filedescr pd, void* recbuf ) {
	int2 err=0;
	if ( pd->keys ) {
		int2 i=0;
		uint4 dummy=0;
		void* keybuf=malloc( pd->hdr->reclen );
		for ( ; i<pd->hdr->numbkeys && !err; i++ ) {
			int hf=pd->keys[i].hf;
			if ( hf>0 && !(pd->keys[i].keysect->comptype&KEYTYP_DUPSW) ) {
				err=key_fill( pd, (int2)(i+1), recbuf, keybuf );
				if ( !err ) err=key_find( hf, keybuf, &dummy, GET_EQ );
				if ( err!=ERR_NOT_FOUND ) {
					if ( !err ) err=ERR_KEYVIOL;
				} else err=ERR_NONE;
			}
		}
		free( keybuf );
	}
	return err;
}

int2 cla_put( pcla_filedescr pd, void* recbuf, uint4* rec ) {
	if ( pd && pd->hdr && recbuf && rec ) {
		int2 err=cla_lockfamily( pd, 1 );
		if ( !err ) {
			err=cla_refresh( pd, 0 );
			if ( !err ) err=cla_queryunique( pd, recbuf );
			if ( !err ) {
				/* write record in .dat */
				if ( *rec==0 ) {  /* new record */
					err=dat_freerec( pd, rec );
					if ( !err ) err=dat_recput_phys( pd, *rec, recbuf );
					if ( !err ) err=cla_refresh( pd, 1 );
				} else { /* updated record - delete keys first */
					int1* recbuf1=(int1*)malloc( pd->hdr->reclen );
					err=dat_recget_phys( pd, *rec, recbuf1 );
					if ( !err ) err=cla_allkeys( pd, recbuf1, *rec, 1 );
					if ( !err ) err=dat_recput_phys( pd, *rec, recbuf );
					free( recbuf1 );
				}
				/* write new key's */
				if ( !err ) err=cla_allkeys( pd, recbuf, *rec, 0 );
			}
			{
				int2 err1=cla_lockfamily( pd, 0 );
				if ( !err ) err=err1;
			}
		}
		return err;
	} else return ERR_INVALID_POINTER;
}

int2 cla_delete( pcla_filedescr pd, uint4 rec ) {
	if ( pd && pd->hdr ) {
		int2 err=cla_lockfamily( pd, 1 );
		if ( !err ) {
			int1* recbuf=(int1*)malloc( pd->hdr->reclen );
			err=cla_refresh( pd, 0 );
			if ( !err ) err=dat_recget_phys( pd, rec, recbuf );
			if ( !err ) err=cla_allkeys( pd, recbuf, rec, 1 );
			if ( !err ) {
				((pcla_hdr_rec)recbuf)->rhd|=RHD_IS_DEL;
				pd->hdr->numdels++;
				((pcla_hdr_rec)recbuf)->rptr=pd->hdr->freerec;
				pd->hdr->freerec=rec;
				err=dat_recput_phys( pd, rec, recbuf );
			}
			if ( !err ) err=cla_refresh( pd, 1 );
			free( recbuf );
			{
				int2 err1=cla_lockfamily( pd, 0 );
				if ( !err ) err=err1;
			}
		}
		return err;
	} else return ERR_INVALID_POINTER;
}
