#ifndef __CLA_STRU_H
#define __CLA_STRU_H
#include <fix_type.h>
#ifdef __cplusplus
  extern "C" {
#endif

/* file header */
/*-----------------sfattr field structure-----------------
        bit 0 - file is locked
        bit 1 - file is owned
        bit 2 - records are encrypted
        bit 3 - memo file exists
        bit 4 - file is compressed
        bit 5 - reclaim deleted records
        bit 6 - file is read only
        bit 7 - file may be created
--------------------------------------------------*/
#define SFATR_LOCKED   (0x01)
#define SFATR_OWNED	   (0x02)
#define SFATR_ENCRYPT  (0x04)
#define SFATR_MEMO     (0x08)
#define SFATR_COMPRESS (0x10)
#define SFATR_RECLAIM  (0x20)
#define SFATR_RDONLY   (0x40)
#define SFATR_CREATE   (0x80)

typedef struct str_cla_hdr_file {
  uint2 filesig;         /* file signature */
	uint2 sfatr;
  uint1 numbkeys;   /* number of keys in file */
  uint4 numrecs;    /* number of records in file */
  uint4 numdels;    /* number of deleted records */
  uint2 numflds;         /* number of fields */
  uint2 numpics;         /* number of pictures */
  uint2 nummars;         /* number of array descriptors */
  uint2 reclen;          /* record length (including record header) */
  uint4 offset;     /* start of data area */
  uint4 logeof;     /* logical end of file */
  uint4 logbof;     /* logical beginning of file */
  uint4 freerec;    /* first usable deleted record */
  uint1 recname[12];/* record name without prefix */
  uint1 memnam[12]; /* memo name without prefix */
  uint1 filpre[3];  /* file name prefix */
  uint1 recpre[3];  /* record name prefix */
  uint2 memolen;         /* size of memo */
  uint2 memowid;         /* column width of memo */
  uint4 reserved;   /* reserved */
  uint4 chgtime;    /* time of last change */
  uint4 chgdate;    /* date of last change */
  uint2 reserved2;       /* reserved */
} cla_hdr_file;
typedef cla_hdr_file* pcla_hdr_file;

/* field header */
typedef struct str_cla_hdr_field {
  uint1 fldtype;  /* type of field */
  int1 fldname[16];       /* name of field */
  uint2 foffset;       /* offset into record */
  uint2 length;        /* length of field */
  uint1 decsig;   /* significance for decimals */
  uint1 decdec;   /* number of decimal places */
  uint2 arrnum;        /* array number */
  uint2 picnum;        /* picture number */
} cla_hdr_field;
typedef cla_hdr_field* pcla_hdr_field;

/*-----------------fldtype-----------------
        1 - LONG
        2 - REAL
        3 - STRING
        4 - STRING WITH PICTURE TOKEN
        5 - BYTE
        6 - SHORT
        7 - GROUP
        8 - DECIMAL
--------------------------------------------------*/
enum fld_type {
	fld_long=1,        /* 1 - LONG */
	fld_real=2,        /* 2 - REAL */
	fld_string=3,      /* 3 - STRING */
	fld_str_pict=4,    /* 4 - STRING WITH PICTURE TOKEN */
	fld_byte=5,        /* 5 - BYTE */
	fld_short=6,       /* 6 - SHORT */
	fld_group=7,       /* 7 - GROUP */
	fld_decimal=8      /* 8 - DECIMAL */
};

/* keys & indexes */

typedef struct str_cla_keysect {
	uint1 numcomps; /* number of components for key */
	int1 keynams[16];       /* name of this key */
	uint1 comptype; /* type of composite */
	uint1 complen;  /* length of composite */
} cla_keysect;
typedef cla_keysect* pcla_keysect;
typedef cla_keysect* pcla_hdr_key;

typedef struct str_cla_keypart {
	uint1 fldtype;  /* type of field */
	uint2 fldnum;        /* field number */
	uint2 elmoff;        /* record offset of this element */
	uint1 elmlen;   /* length of element */
} cla_keypart;
typedef cla_keypart* pcla_keypart;
/* pictures */
typedef struct str_cla_hdr_pict {
	uint2 piclen;
	int1    picstr[256];
} cla_hdr_pict;
typedef cla_hdr_pict* pcla_hdr_pict;
/* arrays */
typedef struct str_cla_arrsect {
  uint2 numdim;/* dims for current field */
  uint2 totdim;/* total number of dims for field */
  uint2 elmsiz;/* total size of current field */
} cla_arrsect;
typedef cla_arrsect* pcla_arrsect;

typedef struct str_cla_arrpart {
  uint2 maxdim;/* number of dims for array part */
  uint2 lendim;/* length of field */
} cla_arrpart;
typedef cla_arrpart* pcla_arrpart;
typedef pcla_arrsect pcla_hdr_arr;
/* record */
/* ----------rhd------------
    bit 0 - new record
    bit 1 - old record
    bit 2 - revised record
    bit 4 - deleted record
    bit 6 - record held
*/
#define RHD_IS_NEW (0x01)
#define RHD_IS_OLD (0x02)
#define RHD_IS_REV (0x04)
/*#define RHD_PAD    (0x08)*/
#define RHD_IS_DEL (0x10)
/*#define RHD_PAD1   (0x20)*/
#define RHD_IS_HLD (0x40)
/*#define RHD_PAD2   (0x80)*/
typedef struct str_cla_hdr_rec {
	uint1 rhd;/* record header type and status */
  uint4 rptr;/* pointer for next deleted record
                        or memo if active */
} cla_hdr_rec;
typedef cla_hdr_rec* pcla_hdr_rec;

#ifdef __cplusplus
  }
#endif
#endif  /* __CLA_STRU_H */
