#include "globals.hpp"


enum DBTYPES { DBNOTYPE, DBBYTE, DBSTRING, DBLONG, DBSHORT, DBDECIMAL, DBREAL, DBMEMO };

class ISAMFile;

struct FieldDescriptor
	{
	WORD Order;
	WORD Len;
	WORD Precision;
	DBTYPES Type;
	};

struct KeyDescriptor {
	KeyDescriptor() : keyNum(0),numComp(0),comp(0),isDup(FALSE),isOpt(FALSE),
			isUpp(FALSE),isAcc(TRUE) {}
	Boolean SetProperty( ISAMFile*, WORD );
	~KeyDescriptor() { delete [] comp; }

	WORD keyNum;
	WORD numComp;
	WORD* comp;
	Boolean isDup;
	Boolean isOpt;
	Boolean isUpp;
	Boolean isAcc;
};

struct FileDescriptor {
	Boolean memo;
	Boolean reclaim;
	BYTE numbkeys;   /* number of keys in file */
	DWORD numrecs;    /* number of records in file */
	DWORD numdels;    /* number of deleted records */
	WORD numflds;         /* number of fields */
	WORD reclen;          /* record length (including record header) */
	WORD memolen;         /* size of memo */
	WORD memowid;         /* column width of memo */
};


class DBField
	{
	private:
		BYTE *inbf;
	protected:
		ISAMFile *fl;
		ERRORS Error;
		FieldDescriptor fd;
	public:
		DBField();
		virtual ~DBField();
		ERRORS GetError()  							{ return Error; }
		void ClearError()							{ Error = NOERROR; }
		Boolean SetProperty( ISAMFile*, WORD );
		Boolean SetProperty( ISAMFile*, char* );
		//accessors
		virtual Boolean SetAsInt( void*, int );
		virtual Boolean SetAsLong( void*, long );
		virtual Boolean SetAsDouble( void*, double );
		virtual Boolean SetAsString( void*, char* );
		virtual Boolean SetAsPtr( void*, void*, WORD );
		virtual Boolean SetAsByte( void*, BYTE );
		virtual Boolean GetAsInt( void*, int* );
		virtual Boolean GetAsLong( void*, long* );
		virtual Boolean GetAsDouble( void*, double* );
		virtual Boolean GetAsString( void*, char*, WORD );
		virtual Boolean GetAsPtr( void*, void*, WORD );
		virtual Boolean GetAsByte( void*, BYTE* );
		virtual Boolean Get( void* rec, void* ) = 0;
		virtual Boolean Set( void* rec, void* ) = 0;
	};


class ISAMFile
	{
	private:
	protected:
		ERRORS Error;
		DWORD SPtr;
		DWORD IPtr;
	public:
		ISAMFile();
		virtual ~ISAMFile();
		ERRORS GetError()  							{ return Error; }
		virtual char *GetErrorString() = 0;
		void ClearError()							{ Error = NOERROR; }
		//standart
		virtual Boolean FieldByName( WORD*, char* ) = 0;
		virtual Boolean KeyByName( WORD*, char* ) = 0;
		virtual Boolean NameByField( char*, WORD, WORD ) = 0;
		virtual Boolean NameByKey( char*, WORD, WORD ) = 0;
		virtual Boolean GetFieldDescriptor( FieldDescriptor*, WORD ) = 0;
		virtual Boolean GetKeyDescriptor( KeyDescriptor*, WORD ) = 0;
		virtual Boolean GetFileDescriptor( FileDescriptor* ) = 0;
		virtual Boolean GetRecordLen( WORD* ) = 0;
		virtual Boolean Open( char* ) = 0;
		virtual Boolean Create( char* ) = 0;
		virtual Boolean Close() = 0;
		virtual Boolean SetFirst( WORD = 0 ) = 0;
		virtual Boolean SetLast( WORD = 0 ) = 0;
		virtual Boolean SetBefore( void*, WORD, DWORD = 0 ) = 0;
		virtual Boolean SetAfter( void*, WORD, DWORD = 0 ) = 0;
		virtual Boolean Get( void*, DWORD, WORD = 0 ) = 0;
		virtual Boolean Next( void* ) = 0;
		virtual Boolean Previous( void* ) = 0;
		virtual Boolean Add( void* ) = 0;
		virtual Boolean Put( void* ) = 0;
		virtual Boolean Delete() = 0;
		virtual Boolean FieldCount( WORD* ) =0;
		virtual Boolean KeyCount( WORD* ) =0;
		virtual Boolean GetMemo( void*, WORD, void* ) =0;
		virtual Boolean SetMemo( void*, WORD, void* ) =0;

	};