/*#define DEBUG_KEYNODE*/
#include <stdlib.h>
#include <io1.h>
#ifdef __OS2__
#include <string.h>
#include <malloc.h>
#else
#include <mem.h>
#include <alloc.h>
#endif
#include <errno.h>
#ifdef DEBUG_KEYNODE
#include <stdio.h>
#endif /* DEBUG_KEYNODE */
#include <cla_err.h>
#include <key_util.h>
/*----------------------------------------------------------------------------
int2 key_gethdr( int hf, pkey_hdr phdr );
int2 key_puthdr( int hf, pkey_hdr phdr );
int2 key_getnode( int hf, uint4 n, pkey_node buf );
int2 key_putnode( int hf, uint4 n, pkey_node buf );
int2 key_next_node( int hf, pkey_node pnode );
int2 key_prev_node( int hf, pkey_node pnode );
int2 key_findnode( int hf, void* buf, pkey_node pnode, uint4* pn, int2* pi, int2* pcmp );
int2 key_updatechilds( int hf, pkey_node pnode, uint4 nn, int2 kl  );
----------------------------------------------------------------------------*/

#ifdef DEBUG_KEYNODE
static int2 keylen=0;
static int1 s[250];

static void printnode( uint4 n, pkey_node pnode, int1* action, int2 err ) {
	int2 i=0;
	/*write( hLog, s, strlen( s ) );*/
	printf( "\nNode\t:%ld,\t:%s,\terror\t:%d", n, action, err );
  printf( "\n*******************" );
  printf( "\n*   %6.6ld  (%3d)*", pnode->ulink, (int2)pnode->keycnt );
	printf( "\n*******************" );
	printf( "\n*%6.6ld * %6.6ld*", pnode->blink, pnode->flink );
	printf( "\n*******************" );
	printf( "\n{" );
	for ( ;i<pnode->keycnt; i++ ) {
		pkey_entry pe=(pkey_entry)&(((int1*)(&pnode[1]))[i*keylen]);
		printf( " [%d]=%ld", i, pe->relrec );
	}
	printf( " }" );
}

static void printheader( pkey_hdr ph, int1* action, int2 err ) {
	printf( "\nKey header\t:%s,\terror:\t%d\n", action, err );
	printf( "\nRoot\t\t:%4ld\t\tUnused\t\t:%4ld", ph->root, ph->unused );
	printf( "\nKey entries\t:%4ld", ph->numkent );
	printf( "\nNodes\t\t:%4ld\t\tLast\t\t:%4ld", ph->numnode, ph->lastnod );
	printf( "\nEof\t\t:%4ld\t\tBof\t\t:%4ld", ph->keyeof, ph->keybof );
	printf( "\nComponents\t:%4d\t\tKeys/node\t:%4d", (int2)ph->numcmps, (int2)ph->keynode );
	printf( "\nKeylen\t\t:%4d\t\tLevels\t\t:%4d", ph->keylen, ph->numlvls );
	printf( "\nType\t:%s\t%s\t%s\t%s\t%s",
			(ph->keytyp&KEYTYP_FTYP ? "Index" : "  Key"),
			(ph->keytyp&KEYTYP_UPRSW ? "  Dupl" : "Unique"),
			(ph->keytyp&KEYTYP_DUPSW ? "Upper" : "Mixed"),
			(ph->keytyp&KEYTYP_OPTSW ? "  Fill" : "NoFill"),
			(ph->keytyp&KEYTYP_LOKSW ? "Locked" : "") );
	printf( "\n" );
}
#endif /* DEBUG_KEYNODE */

int2 key_gethdr( int hf, pkey_hdr phdr ) {
	int2 err=0;
	if ( hf>0 && phdr ) {
		if ( 0==lseek( hf, 0, SEEK_SET )
				&& sizeof(key_hdr)==read( hf, phdr, sizeof(key_hdr) ) ) {
		} else {
#ifdef DEBUG_KEYNODE
			perror( "key_gethdr" );
#endif /* DEBUG_KEYNODE */
			err=( errno==EACCES ) ? ERR_ACCESS : ERR_READ;
		}
	} else err=ERR_INVALID_POINTER;
#ifdef DEBUG_KEYNODE
keylen=phdr->keylen;
printheader( phdr, "READ", err );
#endif /* DEBUG_KEYNODE */
	return err;
}

int2 key_puthdr( int hf, pkey_hdr phdr ) {
	int2 err=0;
	if ( hf>0 && phdr ) {
		if ( 0==lseek( hf, 0, SEEK_SET )
				&& sizeof(key_hdr)==write( hf, phdr, sizeof(key_hdr) ) ) {
		} else {
#ifdef DEBUG_KEYNODE
			perror( "key_puthdr" );
#endif /* DEBUG_KEYNODE */
			err=( errno==EACCES ) ? ERR_ACCESS : ERR_WRITE;
		}
	} else err=ERR_INVALID_POINTER;
#ifdef DEBUG_KEYNODE
keylen=phdr->keylen;
printheader( phdr, "WRITE", err );
#endif /* DEBUG_KEYNODE */
	return err;
}

int2 key_getnode( int hf, uint4 n, pkey_node buf ) {
	int2 err=0;
	if ( hf>0 && n>0 && buf ) {
		if ( (n*sz_node==lseek( hf, n*sz_node, SEEK_SET ))
				&& (sz_node==read( hf, buf, sz_node )) ) {
		} else {
#ifdef DEBUG_KEYNODE
			perror( "key_getnode" );
#endif /* DEBUG_KEYNODE */
			err=( errno==EACCES ) ? ERR_ACCESS : ERR_READ;
		}
	} else err=ERR_INVALID_POINTER;
#ifdef DEBUG_KEYNODE
printnode( n, buf, "READ", err );
#endif /* DEBUG_KEYNODE */
	return err;
}

int2 key_putnode( int hf, uint4 n, pkey_node buf ) {
	int2 err=0;
	if ( hf>0 && n>0 && buf ) {
		if ( (n*sz_node==lseek( hf, n*sz_node, SEEK_SET ))
				&& (sz_node==write( hf, buf, sz_node )) ) {
		} else {
#ifdef DEBUG_KEYNODE
			perror( "key_putnode" );
#endif /* DEBUG_KEYNODE */
			err=( errno==EACCES ) ? ERR_ACCESS : ERR_WRITE;
		}
	} else err=ERR_INVALID_POINTER;
#ifdef DEBUG_KEYNODE
printnode( n, buf, "WRITE", err );
#endif /* DEBUG_KEYNODE */
	return err;
}

int2 key_next_node( int hf, pkey_node pnode ) {
	if ( !pnode ) return ERR_INVALID_POINTER;
	if ( pnode->flink ) {
		return key_getnode( hf, pnode->flink, pnode );
	} else return ERR_NOT_FOUND;
}

int2 key_prev_node( int hf, pkey_node pnode ) {
	if ( !pnode ) return ERR_INVALID_POINTER;
	if ( pnode->blink ) {
		return key_getnode( hf, pnode->blink, pnode );
	} else return ERR_NOT_FOUND;
}

/* find a node where given key is or should be
 hf    - in  : file handle
 buf   - in  : key
 pnode - out : buffer filled with node found
 pn    - out : addr. of phys. number of node
 pi    - out : addr. of index in node
 pcmp  - out : addr. of compare result

 if there is no error condition, '*pn' & '*pi' always contain valid
 values, i.e. corresponding element already exist.
 '*pnode' contain node with phys. number '*pn'
 If '*pcmp'==0, key found ( fist occurance );
 if '*pcmp'<0, key should be before pointed by '*pn' & '*pi';
 if '*pcmp'>0, key should be after ( this happens only when key is
 larger than maximal one in file.
 */
int2 key_findnode( int hf, void* buf, uint4 rec, pkey_node pnode,
	uint4* pn, int2* pi, int2* pcmp ) {
	if ( hf>0 && buf && pnode && pn && pi && pcmp ) {
		pkey_hdr ph=(pkey_hdr)malloc( sizeof( key_hdr ) );
		int2 err=key_gethdr( hf, ph );
		uint2 lvl=1;
		int2 i=0, cmp=1; /* initial cmp>0 for correct work with empty index */
		uint4 n=ph->root;
		if ( !err ) err=key_getnode( hf, n, pnode );
		while ( lvl<=ph->numlvls && i<pnode->keycnt && !err ) {
			pkey_entry pe=(pkey_entry)&(((int1*)(&pnode[1]))[i*ph->keylen]);
			cmp=(int2)memcmp( buf, pe->key, ph->keylen-4 );
			if ( cmp>0 ) i++;
			else {
				if ( lvl<ph->numlvls ) { /* it's link */
					n=pe->relrec;
					err=key_getnode( hf, n, pnode );
					i=0;
				}
				lvl++;
			}
		}
		if ( !err ) {
			if ( cmp>0 ) {
				/* we are at root and not found proper place */
				/* this key should be after last entry in last node */
				n=ph->lastnod;
				err=key_getnode( hf, n, pnode );
				i=(int2)(pnode->keycnt-1);
			} else if ( lvl>ph->numlvls && rec>0 && cmp==0 ) {
				/* we are at leaf level and have found 1-st duplicate */
				while ( !err && cmp>=0 ) {
					/* leaf elements with duplicate keys ordered by phys. record number */
					pkey_entry pe=(pkey_entry)&(((int1*)(&pnode[1]))[i*ph->keylen]);
					cmp=(int2)memcmp( buf, pe->key, ph->keylen-4 );
					if ( cmp==0 ) cmp=rec<pe->relrec ? (int2)-1 : rec>pe->relrec ? 1 : 0;
					if ( cmp>0 ) i++;
					if ( i>=pnode->keycnt ) {
						if ( pnode->flink ) {
							n=pnode->flink;
							i=0;
							err=key_getnode( hf, n, pnode );
						} else {
							i=(int2)(pnode->keycnt-1);
							break;
						}
					}
				}
			}
		}
		if ( !err ) {
			/* first node >= (buf, rec) found */
			*pn=n;
			*pi=i;
			*pcmp=cmp;
		}
		free( ph );
		return err;
	} else return ERR_INVALID_POINTER;
}

/* find phys. rec. number
 --------action-------------
 bits  | bitmask   | meaning
 0     | K_INEXACT | if not exact match, look for next/prev
 1     | K_PREV    | look for prev
 2     | K_OPPOSIT | look for opposite side


            ********** buf point here
            
 ..........PTN.....    record in key sequence

 if ( K_INEXACT set ) {
	 if ( record T exist ) return T;
	 else {
		 look for ( K_PREV set ? P : N );
		 if ( found ) return record found;
		 else if ( K_OPPOSIT set ) {
			 look for opposit side;
			 if ( found ) return record found;
			 else return not found; #( no records in table )
		 } else return not found;
	 }
 } else if ( record T exist ) {
	 if ( K_PREV set ) {
		 return ( K_OPPOSIT set ? N : P );
	 } else return T;
 } else return not found;

 Shortkey:
 to look for...               use 'action'

 exact key                    0
 next/prev                    K_PREV|K_OPPOSIT / K_PREV
 key or next/prev             K_INEXACT / K_INEXACT|K_PREV

 */

/* sync childs with new parent
 done by writting new links for all childs

 */
int2 key_updatechilds( int hf, pkey_node pnode, uint4 nn, int2 kl, int2 i0, int2 i1 ) {
	int2 i=i0;
	pkey_node pnode1=(pkey_node)malloc( sz_node );
	int2 err=0;
	if ( i1<0 ) i1=pnode->keycnt;
	for ( ; !err && i<i1; i++ ) {
		pkey_entry pe=((pkey_entry)&(((int1*)(&(pnode)[1]))[i*kl]));
		err=key_getnode( hf, pe->relrec, pnode1 );
		pnode1->ulink=nn;
		if ( !err ) err=key_putnode( hf, pe->relrec, pnode1 );
	}
	free( pnode1 );
	return err;
}
