#include <string.h>
#include <malloc.h>
#include <stdio.h>
#include <os2.h>
#define INCL_RXSHV
#define INCL_RXFUNC
#include <rexxsaa.h>
#include <cla_file.h>
#include <cla_dat.h>
#include <key_stru.h>
#include <rxshv.h>

RexxFunctionHandler RxClaInfo;

#define  INVALID_ROUTINE 40            /* Raise Rexx error           */
#define  VALID_ROUTINE    0            /* Successful completion      */

/* Get information for clarion data file
RxClaInfo( fname, stem. )
Fill stem with stem. + :
!DATA.!NAME
!DATA.!SFATR
!DATA.!NUMBKEYS
!DATA.!NUMRECS
!DATA.!NUMDELS
!DATA.!NUMFLDS
!DATA.!NUMPICS
!DATA.!NUMMARS
!DATA.!RECLEN
!DATA.!LOGEOF
!DATA.!LOGBOF
!DATA.!LOGBOF
!DATA.!OFFSET
!DATA.!FREEREC
!DATA.!RECNAME
!DATA.!MEMNAM
!DATA.!FILPRE
!DATA.!RECPRE
!DATA.!MEMOLEN
!DATA.!MEMOWID
!DATA.!CHGDATETIME
!FIELDS.0
!FIELDS.%d.!NAME
!FIELDS.%d.!FOFFSET
!FIELDS.%d.!LENGTH
!FIELDS.%d.!DECSIG
!FIELDS.%d.!DECDEC
!FIELDS.%d.!ARRNUM
!FIELDS.%d.!PICNUM
!FIELDS.%d.!FLDTYPE
!KEYS.0
KEYS.%d.!NAME
!KEYS.%d.!NUMCOMPS
!KEYS.%d.!COMPLEN
!KEYS.%d.!COMPTYPE
!KEYS.%d.!COMPLIST
*/

ULONG RxClaInfo(CHAR *name, ULONG numargs, RXSTRING args[],
                    CHAR *queuename, RXSTRING *retstr) {
	/* parse parameters */
	if ( numargs==2 ) {
		char* fname=args[0].strptr;
		char* stem=strcpy( malloc(256), args[1].strptr );
		char* ss=malloc(256);
		cla_filedescr fd;
		SHVBLOCK* shv=0;                  /* block to pass to REXX       */
		int2 err=0;
	  APIRET rc=0;                       /* return code from calls      */
		memset( &fd, 0, sizeof(cla_filedescr) );
    err=cla_open( &fd, fname, REPAIR_KEEP_BROKEN );
		if ( !err ) {
			char* s=strchr( stem, '\0' );
			sprintf( s, "!DATA.!NAME" );
			sprintf( ss, "%s", fd.fname );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			/* Header */
			sprintf( s, "!DATA.!SFATR" );
			sprintf( ss,
				"%cLock %cOwned %cEncrypt %cMemo %cCompress %cReclaim %cRdOnly %cCreate",
				fd.hdr->sfatr&SFATR_LOCKED ? '+' : '-',
				fd.hdr->sfatr&SFATR_OWNED ? '+' : '-',
				fd.hdr->sfatr&SFATR_ENCRYPT ? '+' : '-',
				fd.hdr->sfatr&SFATR_MEMO ? '+' : '-',
				fd.hdr->sfatr&SFATR_COMPRESS ? '+' : '-',
				fd.hdr->sfatr&SFATR_RECLAIM ? '+' : '-',
				fd.hdr->sfatr&SFATR_RDONLY ? '+' : '-',
				fd.hdr->sfatr&SFATR_CREATE ? '+' : '-'
				 );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!NUMBKEYS" );
			sprintf( ss, "%d", (int)(fd.hdr->numbkeys) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!NUMRECS" );
			sprintf( ss, "%ld", (long)(fd.hdr->numrecs) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!NUMDELS" );
			sprintf( ss, "%ld", (long)(fd.hdr->numdels) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!NUMFLDS" );
			sprintf( ss, "%d", (int)(fd.hdr->numflds) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!NUMPICS" );
			sprintf( ss, "%d", (int)(fd.hdr->numpics) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!NUMMARS" );
			sprintf( ss, "%d", (int)(fd.hdr->nummars) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!RECLEN" );
			sprintf( ss, "%d", (int)(fd.hdr->reclen) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!LOGEOF" );
			sprintf( ss, "%ld", (long)(fd.hdr->logeof) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!LOGBOF" );
			sprintf( ss, "%ld", (long)(fd.hdr->logbof) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!LOGBOF" );
			sprintf( ss, "%ld", (long)(fd.hdr->logbof) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!OFFSET" );
			sprintf( ss, "%ld", (long)(fd.hdr->offset) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!FREEREC" );
			sprintf( ss, "%ld", (long)(fd.hdr->freerec) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!RECNAME" );
			sprintf( ss, "%12.12s", fd.hdr->recname );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!MEMNAM" );
			sprintf( ss, "%12.12s", fd.hdr->memnam );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!FILPRE" );
			sprintf( ss, "%3.3s", fd.hdr->filpre );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!RECPRE" );
			sprintf( ss, "%3.3s", fd.hdr->recpre );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!MEMOLEN" );
			sprintf( ss, "%d", (int)(fd.hdr->memolen) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			sprintf( s, "!DATA.!MEMOWID" );
			sprintf( ss, "%d", (int)(fd.hdr->memowid) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			{
				cla_date d;
				cla_time t;
				cla_date_decode( &d, fd.hdr->chgdate );
				cla_time_decode( &t, fd.hdr->chgtime );
				sprintf( s, "!DATA.!CHGDATETIME" );
				sprintf( ss, "%d/%d/%d %d:%d:%d.%d",
					(int)d.day, (int)d.month, (int)d.year,
					(int)t.hour, (int)t.min, (int)t.sec, (int)t.hundreds );
				shv=ShvNew( shv, stem, ss, RXSHV_SET );
			}
			/* Fields */
			sprintf( s, "!FIELDS.0" );
			sprintf( ss, "%d", (int)(fd.hdr->numflds) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			{
				int i=0;
				for ( ; i<fd.hdr->numflds; i++ ) {
					sprintf( s, "!FIELDS.%d.!NAME", i+1 );
					sprintf( ss, "%16.16s", fd.hdr_field[i].fldname );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!FIELDS.%d.!FOFFSET", i+1 );
					sprintf( ss, "%d", (int)(fd.hdr_field[i].foffset) );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!FIELDS.%d.!LENGTH", i+1 );
					sprintf( ss, "%d", (int)(fd.hdr_field[i].length) );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!FIELDS.%d.!DECSIG", i+1 );
					sprintf( ss, "%d", (int)(fd.hdr_field[i].decsig) );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!FIELDS.%d.!DECDEC", i+1 );
					sprintf( ss, "%d", (int)(fd.hdr_field[i].decdec) );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!FIELDS.%d.!ARRNUM", i+1 );
					sprintf( ss, "%d", (int)(fd.hdr_field[i].arrnum) );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!FIELDS.%d.!PICNUM", i+1 );
					sprintf( ss, "%d", (int)(fd.hdr_field[i].picnum) );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!FIELDS.%d.!FLDTYPE", i+1 );
					cla_fldtype( &fd, (int2)(i+1), ss );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
				}
			}
			/* Keys */
			sprintf( s, "!KEYS.0" );
			sprintf( ss, "%d", (int)(fd.hdr->numbkeys) );
			shv=ShvNew( shv, stem, ss, RXSHV_SET );
			if ( fd.hdr->numbkeys>0 && fd.keys ) {
				int i=0;
				for ( ; i<fd.hdr->numbkeys; i++ ) {
					sprintf( s, "!KEYS.%d.!NAME", i+1 );
					sprintf( ss, "%16.16s", fd.keys[i].keysect->keynams );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!KEYS.%d.!NUMCOMPS", i+1 );
					sprintf( ss, "%d", (int)(fd.keys[i].keysect->numcomps) );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!KEYS.%d.!COMPLIST", i+1 );
					{
						char* s1=ss;
						int j=0, i1=(int)(fd.keys[i].keysect->numcomps);
            pcla_keypart pp=(pcla_keypart)(void*)(&fd.keys[i].keysect[1]);
						*s1='\0';
						for ( ; j<i1; j++ ) {
							sprintf( s1, " %d", (int)(pp[j].fldnum) );
							s1=strchr( s1, '\0' );
						}
					}
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!KEYS.%d.!COMPLEN", i+1 );
					sprintf( ss, "%d", (int)(fd.keys[i].keysect->complen) );
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
					sprintf( s, "!KEYS.%d.!COMPTYPE", i+1 );
					sprintf( ss, "%s %cDup %cUpr %cOpt %cLock",
						fd.keys[i].keysect->comptype&KEYTYP_FTYP ? "Key" : "Index",
						fd.keys[i].keysect->comptype&KEYTYP_DUPSW ? '+' : '-',
						fd.keys[i].keysect->comptype&KEYTYP_UPRSW ? '+' : '-',
						fd.keys[i].keysect->comptype&KEYTYP_OPTSW ? '+' : '-',
						fd.keys[i].keysect->comptype&KEYTYP_LOKSW ? '+' : '-'
						);
					shv=ShvNew( shv, stem, ss, RXSHV_SET );
				}
			}
			/* Arrays */

			/* Pictures */
      rc = RexxVariablePool(shv) & 0xFFFFFFFE;
			err=cla_close( &fd );
		}
		if ( !err && !rc ) {
			retstr->strptr[0]=0;
			retstr->strlength=0;
		} else {
			if ( err ) {
				cla_errstring( err, retstr->strptr, 256 );
			} else {
				sprintf( retstr->strptr, "Rexx :%s%s%s%s%s%s",
					rc&RXSHV_NEWV ? " NEWV" : "",
					rc&RXSHV_LVAR ? " LVAR" : "",
					rc&RXSHV_TRUNC ? " TRUNC" : "",
					rc&RXSHV_BADN ? " BADN" : "",
					rc&RXSHV_MEMFL ? " MEMFL" : "",
					rc&RXSHV_BADF ? " BADF" : ""
				);
			}
			retstr->strlength=strlen( retstr->strptr );
		}
		free( ss );
		ShvDel( shv );
		free( stem );
		return VALID_ROUTINE;
	} else return INVALID_ROUTINE;
}

