#include <string.h>
#include <malloc.h>
#include <stdio.h>
#include <os2.h>
#define INCL_RXSHV
#define INCL_RXFUNC
#include <rexxsaa.h>
#include <cla_file.h>
#include <cla_dat.h>
#include <key_stru.h>
#include <cla_key.h>
#include <cla_str.h>
#include <rxshv.h>
/* RxClaGet( file, "stem.", key ,N|P|F|x )
 get record by key number and operation
 if key==0, by physical order
 operation : N(ext), P(rev), F(ind), x - record number */
RexxFunctionHandler RxClaGet;

#define  INVALID_ROUTINE 40            /* Raise Rexx error           */
#define  VALID_ROUTINE    0            /* Successful completion      */

/* set digit of d[i] of total size 'n' to x */
static void dec_set( BYTE* d, BYTE n, BYTE i, BYTE x ) {
	d[(i+(n^1))/2]|=(n^i)&1 ? (x<<4)&0xf0 : x&0x0f ;
}

static void str2dec( BYTE *d, BYTE n, BYTE m, char* str )	{
	char* dot=strchr( str, '.' );
	int k1=0;
	n+=m;
	if ( !dot ) dot=strchr( str, '\0' );
	k1=dot-str;
	memset( d, 0, (size_t)(n+1) );
	if ( *str=='-' ) k1--;
	/* 'k1' digits before dot */
	if ( k1<=n-m ) {
		char* s=--dot;
		if ( *str=='-' ) {
			str++;
			dec_set( d, n, 0, 1 );
		}
		for ( ; s>=str; s-- ) {
			dec_set( d, (BYTE)(n), (BYTE)((s-dot)+1+(n-m)), (BYTE)(*s) );
		}
		if ( *dot ) {
			for ( s=++dot; *s && (s-dot)<=m ; s++ ) {
				dec_set( d, (BYTE)(n), (BYTE)((s-dot)+(n-m)), (BYTE)(*s) );
			}
		}
	} /* else error */
}

static void dec2str( BYTE *d, BYTE n, BYTE m, char* str )	{
	n+=m;
	if( n&1 ) {
		if( *d&0xf0 ) *str++='-';
		*str++=(*d++&0x0f)+'0';
		n--;
	} else {
		if( *d++&0x0f ) *str++='-';
	}
	while ( n ) {
		if ( n==m ) *str++='.';
		*str++=(( n&1 ? *d++ : (*d)>>4 )&0x0f)+'0';
		n--;
	}
	*str='\0';
}

static APIRET stem2rec( pcla_filedescr pd, void* recbuf, char* stem ) {
	int i=0, i1=pd->hdr->numflds;
	char* s=malloc(256);
	char* ss=malloc(256);
	SHVBLOCK* shv=0;                  /* block to pass to REXX       */
	APIRET rc=0;                       /* return code from calls      */
	memset( ss, ' ', 255 );
	ss[255]='\0';
	for ( i=i1; i>0; i-- ) { /* for each field */
		sprintf( s, "%s%d", stem, i );
		shv=ShvNew( shv, s, ss, RXSHV_FETCH );
	}
	free( ss );
	ss=0;
	rc = RexxVariablePool(shv) & 0xFFFFFFFE;
	if ( rc==0 ) {
		SHVBLOCK* shv1=shv;
		for ( i=0; i<i1 && shv1; i++, shv1=shv1->shvnext ) { /* for each field */
			void* fld=&(((char*)recbuf)[sizeof( cla_hdr_rec )+pd->hdr_field[i].foffset]);
			ss=shv1->shvvalue.strptr;
			switch ( pd->hdr_field[i].fldtype ) {
				case 1:  /* LONG */
				sscanf( ss, "%ld", (int4*)fld );
				break;
				case 2:  /* REAL */
				sscanf( ss, "%f", (float*)fld );
				break;
				case 3:  /* STRING */
				case 4:  /* STRING WITH PICTURE TOKEN */
				cla_strcpy1( fld, ss, pd->hdr_field[i].length );
				break;
				case 5:  /* BYTE */
				*(int1*)fld=(int1)(ss[0]-'0');
				break;
				case 6:  /* SHORT */
				sscanf( ss, "%hd", (int2*)fld );
				break;
				case 8:  /* DECIMAL */
				str2dec( (BYTE*)fld, (BYTE)(pd->hdr_field[i].decsig), (BYTE)(pd->hdr_field[i].decdec), ss );
				break;
			}
		}
	}
	ShvDel( shv );
	free( s );
	return rc;
}

static APIRET rec2stem( pcla_filedescr pd, void* recbuf, char* stem, long recno ) {
	int i=0, i1=pd->hdr->numflds;
	char* s=malloc(256);
	char* ss=malloc(256);
	SHVBLOCK* shv=0;                  /* block to pass to REXX       */
	APIRET rc=0;                       /* return code from calls      */
	sprintf( s, "%s!RECNO", stem );
	sprintf( ss, "%ld", recno );
	shv=ShvNew( shv, s, ss, RXSHV_SET );
	sprintf( s, "%s0", stem );
	sprintf( ss, "%d", i1 );
	shv=ShvNew( shv, s, ss, RXSHV_SET );
	for ( ; i<i1; i++ ) { /* for each field */
		void* fld=&(((char*)recbuf)[sizeof( cla_hdr_rec )+pd->hdr_field[i].foffset]);
		*ss='\0';
		switch ( pd->hdr_field[i].fldtype ) {
			case 1:  /* LONG */
			sprintf( ss, "%ld", (long)*(int4*)fld );
			break;
			case 2:  /* REAL */
			sprintf( ss, "%f", *(float*)fld );
			break;
			case 3:  /* STRING */
			case 4:  /* STRING WITH PICTURE TOKEN */
			memcpy( ss, fld, pd->hdr_field[i].length );
			ss[pd->hdr_field[i].length]='\0';
			break;
			case 5:  /* BYTE */
			sprintf( ss, "%hd", (short)*(int1*)fld );
			break;
			case 6:  /* SHORT */
			sprintf( ss, "%hd", (short)*(int2*)fld );
			break;
			case 8:  /* DECIMAL */
			dec2str( (BYTE*)fld, (BYTE)(pd->hdr_field[i].decsig), (BYTE)(pd->hdr_field[i].decdec), ss );
			break;
			default:
			sprintf( ss, "<<Group>>[%d]", (int)pd->hdr_field[i].length );
		}
		sprintf( s, "%s%d", stem, i+1 );
		shv=ShvNew( shv, s, ss, RXSHV_SET );
		/*
		sprintf( s, "%s!%16.16s", stem, pd->hdr_field[i].fldname );
		shv=ShvNew( shv, s, ss, RXSHV_SET );
		*/
	}
	rc = RexxVariablePool(shv) & 0xFFFFFFFE;
	ShvDel( shv );
	free( ss );
	free( s );
	return rc;
}


ULONG RxClaGet(CHAR *name, ULONG numargs, RXSTRING args[],
                    CHAR *queuename, RXSTRING *retstr) {
	/* parse parameters */
	ULONG ret=INVALID_ROUTINE;
	if ( numargs==4 ) {
		char* fname=args[0].strptr;
		char* stem=args[1].strptr;
		cla_filedescr fd;
		int2 err=0;
	  APIRET rc=0;                       /* return code from calls      */
		int key=0;
		uint4 recno=0;
		char op='O';
		memset( &fd, 0, sizeof(cla_filedescr) );
		if ( 1==sscanf( args[2].strptr, "%d", &key ) ) {
			switch ( args[3].strptr[0] ) {
				case 'F':
				case 'N':
				case 'P':
				op=args[3].strptr[0];
				ret=key>0 ? VALID_ROUTINE : INVALID_ROUTINE;
				break;
				default:
				if ( 1==sscanf( args[3].strptr, "%ld", &recno ) ) ret=VALID_ROUTINE;
			}
			if ( ret==VALID_ROUTINE ) {
		    if ( !err ) err=cla_open( &fd, fname, REPAIR_KEEP_BROKEN );
				if ( !err ) {
					void* recbuf=malloc( fd.hdr->reclen );
					if ( key ) {
						if ( op=='O' ) {
							err=key_order( fd.hfile, recno, &recno );
						} else {
							void* keybuf=malloc( fd.keys[key-1].keysect->complen );
							/* fill record */
							rc=stem2rec( &fd, recbuf, stem );
							if ( rc ) err=(int2)(-1);
							if ( !err ) err=key_fill( &fd, key, recbuf, keybuf );
							if ( !err ) err=key_find( fd.hfile, keybuf, &recno,
								op=='F' ? GET_EQ : op=='N' ? GET_MORE : GET_LESS );
							free( keybuf );
						}
					}
					if ( !err ) err=dat_recget_phys( &fd, recno, recbuf );
					if ( !err ) {
						rc=rec2stem( &fd, recbuf, stem, recno );
					}
					free( recbuf );
					{
						int err1=cla_close( &fd );
						if ( !err ) err=err1;
					}
				}
			}
		}
		sprintf( retstr->strptr, "%d : %ld", (int)err, (long)rc );   /* format return code         */		retstr->strlength=strlen( retstr->strptr );
	}
	return ret;
}
