#include <io1.h>
#include <string.h>
#include <cla_err.h>
#include <cla_mem.h>

typedef struct str_hdr_memo {                                                      
  uint2 memsig;   /* memo file signature  = 0x334D */    
  uint4 firstdel; /* first deleted memo block */ 
} memo_hdr;                                                     

typedef struct {                                                    
  uint4 nxtblk;    /* next block for this memo */ 
  uint1 memo[252]; /* memo text */                
} memo_blk;                                                   

static int2 mem_hdr( int hf, memo_hdr* hdr, int2 wr ) {
  int2 err=(int2)(0==lseek( hf, 0, SEEK_SET ) && 
    sizeof(memo_hdr)==(wr ? write( hf, hdr, sizeof(memo_hdr) ) : read( hf, hdr, sizeof(memo_hdr) ) ) ? 0 : wr ? ERR_WRITE : ERR_READ);
  return err;
}

static int2 mem_blk( int hf, memo_blk* blk, uint4 rptr, int2 wr ) {
  uint4 off=(rptr-1)*256+6;
  int2 err=(int2)(off==lseek( hf, off, SEEK_SET ) && 
    sizeof(memo_blk)==(wr ? write( hf, blk, sizeof(memo_blk) ) : read( hf, blk, sizeof(memo_blk) ) ) ? 0 : wr ? ERR_WRITE : ERR_READ);
  return err;
}

int2 cla_memget( int hf, uint4 rptr, uint4* sz, void* buf ) {
  int2 err=(hf>0 && rptr>0 && sz && (*sz==0 || buf) ? ERR_NONE : ERR_INVALID_POINTER);
  if ( !err ) {
    memo_blk blk;
    uint4 sz1=0;
    blk.nxtblk=rptr;
    do {
      err=mem_blk( hf, &blk, blk.nxtblk, 0 );
      if ( !err ) {
        if ( buf ) {
          int n=(int)(*sz-sz1);
          if ( n>252 ) n=252;
          if ( n<=0 ) break;
          memcpy( buf, blk.memo, n );
          buf=(void*)&(((int1*)buf)[n]);
        }
        sz1+=252;
      }
    } while ( !err && blk.nxtblk ); /* enddo */
    if ( !err ) {
      if ( *sz==0 ) *sz=sz1;
    }
  }
  return err;
}

/* delete queue */
static int2 del_blk( int hf, uint4 rptr ) {
  memo_hdr hdr;
  int2 err=mem_hdr( hf, &hdr, 0 );
  if ( !err ) {
    memo_blk blk;
    uint4 rptr1=0;
    blk.nxtblk=rptr;
    do {
      err=mem_blk( hf, &blk, rptr1=blk.nxtblk, 0 );
    } while ( !err && blk.nxtblk ); /* enddo */
    if ( !err ) {
      blk.nxtblk=hdr.firstdel;
      err=mem_blk( hf, &blk, rptr1, 1 );
      if ( !err ) {
        hdr.firstdel=rptr;
        err=mem_hdr( hf, &hdr, 1 );
      }
    }
  }
  return err;
}

static int2 add_blk( int hf, uint4* rptr ) {
  memo_hdr hdr;
  int2 err=mem_hdr( hf, &hdr, 0 );
  if ( hdr.firstdel ) {
    memo_blk blk;
    err=mem_blk( hf, &blk, *rptr=hdr.firstdel, 0 );
    if ( !err ) {
      hdr.firstdel=blk.nxtblk;
      blk.nxtblk=0;
      err=mem_blk( hf, &blk, *rptr, 1 );
      if ( !err ) {
        err=mem_hdr( hf, &hdr, 1 );
      }
    }
  } else {
    int4 off=lseek( hf, 0, SEEK_END )-6;
    if ( off>0 && off%256==0 ) {
      *rptr=(uint4)(off/256+1);
    } else err=ERR_FILE_CORRUPTED;
  } /* endif */
  return err;
}

int2 cla_memset( int hf, uint4* rptr, uint4 sz, void* buf ) {
  int2 err=(hf>0 && rptr && *rptr>0 && (sz==0 || buf) ? ERR_NONE : ERR_INVALID_POINTER);
  if ( !err ) {
    err=del_blk( hf, *rptr );
    if ( !err ) *rptr=0;
    if ( !err && sz>0 && buf ) {
      memo_blk blk;
      uint4 n0=sz/252, n1=sz%252;
      if ( n1==0 ) {
        n1=252;
        n0--;  
      }
      /* 1 block with n1 bytes + n0 full blocks */
      blk.nxtblk=0;
      err=add_blk( hf, rptr );
      if ( !err ) {
        memcpy( blk.memo, &((int1*)buf)[n0*252], n1 );
        err=mem_blk( hf, &blk, *rptr, 1 );
      }
      while ( !err && n0>0 ) {
        n0--;
        blk.nxtblk=*rptr;
        err=add_blk( hf, rptr );
        memcpy( blk.memo, &((int1*)buf)[n0*252], 252 );
        if ( !err ) err=mem_blk( hf, &blk, *rptr, 1 );
      } /* endwhile */
    }
  }
  return err;
}
